
#include <SDL.h>
#include "systemstub.h"

static SDL_Surface *screenSurface;
static uint16 screenPalette[256];
static int screenW, screenH;

void sysInit(int w, int h, const char *title) {
	SDL_Init(SDL_INIT_VIDEO);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_WM_SetCaption(title, NULL);	
	screenW = w;
	screenH = h;
	screenSurface = SDL_SetVideoMode(screenW, screenH, 16, SDL_SWSURFACE);
	memset(screenPalette, 0, sizeof(screenPalette));
}

void sysDestroy() {
	SDL_FreeSurface(screenSurface);
}

void sysSetPalette(const uint8 *palData, int numColors) {
	assert(numColors <= 256);
	for (int i = 0; i < numColors; ++i) {
		const uint8 r = palData[0];
		const uint8 g = palData[1];
		const uint8 b = palData[2];
		screenPalette[i] = SDL_MapRGB(screenSurface->format, r, g, b);
		palData += 3;
	}
}

void sysCopyRect(const uint8 *src, int pitch, int x, int y, int w, int h) {
	SDL_LockSurface(screenSurface);
	uint16 *dst = (uint16 *)screenSurface->pixels + (screenSurface->pitch >> 1) * y + x;
	for (int j = 0; j < h; ++j) {
		for (int i = 0; i < w; ++i) {
			dst[i] = screenPalette[src[i]];
		}
		src += pitch;
		dst += screenSurface->pitch >> 1;
	}
	SDL_UnlockSurface(screenSurface);
	SDL_UpdateRect(screenSurface, x, y, w, h);
}

bool sysDelay(int amount) {
	bool quit = false;
	const uint32 timestampEnd = SDL_GetTicks() + amount;
	SDL_Event ev;
	do {
		while (SDL_PollEvent(&ev)) {
			switch (ev.type) {
			case SDL_QUIT:
				quit = true;
				break;
			default:
				break;
			}
		}
		SDL_Delay(20);
	} while (SDL_GetTicks() < timestampEnd);
	return quit;
}
